# OroApiBundle Documentation

 - [CLI Commands](./commands.md)
 - [Configure Stateless Security Firewalls](./security.md)
 - [Configuration Reference](./configuration.md)
 - [Overview](./configuration.md#overview)
 - [Configuration Structure](./configuration.md#configuration-structure)
   - [**exclude** Option](./configuration.md#exclude-option)
   - [**entity_aliases** Configuration Section](./configuration.md#entity_aliases-configuration-section)
   - [**entities** Configuration Section](./configuration.md#entities-configuration-section)
   - [**fields** Configuration Section](./configuration.md#fields-configuration-section)
   - [**filters** Configuration Section](./configuration.md#filters-configuration-section)
   - [**sorters** Configuration Section](./configuration.md#sorters-configuration-section)
   - [**actions** Configuration Section](./configuration.md#actions-configuration-section)
   - [**status_codes** Configuration Section](./configuration.md#action-status_codes-configuration-section)
   - [**fields** Configuration Section](./configuration.md#action-fields-configuration-section)
   - [**subresources** Configuration Section](./configuration.md#subresources-configuration-section)
   - [**relations** Configuration Section](./configuration.md#relations-configuration-section)
 - [Configuration Extras](./configuration_extra.md)
   - [Overview](./configuration_extra.md#overview)
   - [ConfigExtraInterface](./configuration_extra.md#configextrainterface)
   - [ConfigExtraSectionInterface](./configuration_extra.md#configextrasectioninterface)
   - [Example of configuration extra](./configuration_extra.md#example-of-configuration-extra)
 - [Configuration Extensions](./configuration_extensions.md)
   - [Overview](./configuration_extensions.md#overview)
   - [Creating a Configuration Extension](./configuration_extensions.md#creating-a-configuration-extension)
   - [Add Options to an Existing Configuration Section](./configuration_extensions.md#add-options-to-an-existing-configuration-section)
   - [Add a New Configuration Section](./configuration_extensions.md#add-a-new-configuration-section)
 - [Forms and Validators Configuration](./forms.md)
 - [Documenting API Resources](./documentation.md)
   - [Overview](./documentation.md#overview)
   - [Documentation File Format](./documentation.md#documentation-file-format)
 - [Actions](./actions.md)
   - Public actions
     - [**get** Action](./actions.md#get-action)
     - [**get_list** Action](./actions.md#get_list-action)
     - [**delete** Action](./actions.md#delete-action)
     - [**delete_list** Action](./actions.md#delete_list-action)
     - [**create** Action](./actions.md#create-action)
     - [**update** Action](./actions.md#update-action)
     - [**get_subresource** Action](./actions.md#get_subresource-action)
     - [**update_subresource** Action](./actions.md#update_subresource-action)
     - [**add_subresource** Action](./actions.md#add_subresource-action)
     - [**delete_subresource** Action](./actions.md#delete_subresource-action)
     - [**get_relationship** Action](./actions.md#get_relationship-action)
     - [**update_relationship** Action](./actions.md#update_relationship-action)
     - [**add_relationship** Action](./actions.md#add_relationship-action)
     - [**delete_relationship** Action](./actions.md#delete_relationship-action)
     - [**options** Action](./actions.md#options-action)
   - Auxiliary actions
     - [**customize_loaded_data** Action](./actions.md#customize_loaded_data-action)
     - [**customize_form_data** Action](./actions.md#customize_form_data-action)
     - [**get_config** Action](./actions.md#get_config-action)
     - [**get_relation_config** Action](./actions.md#get_relation_config-action)
     - [**get_metadata** Action](./actions.md#get_metadata-action)
     - [**normalize_value** Action](./actions.md#normalize_value-action)
     - [**collect_resources** Action](./actions.md#collect_resources-action)
     - [**collect_subresources** Action](./actions.md#collect_subresources-action)
     - [**not_allowed** Action](./actions.md#not_allowed-action)
   - [**Context** Class](./actions.md#context-class)
   - [**SubresourceContext** Class](./actions.md#subresourcecontext-class)
   - [Creating New Action](./actions.md#creating-new-action)
 - [The Request Type](./request_type.md)
 - [Processors](./processors.md)
   - [Overview](./processors.md#overview)
   - [Creating a Processor](./processors.md#creating-a-processor)
   - [Processor Conditions](./processors.md#processor-conditions)
   - [Examples of Processor Conditions](./processors.md#examples-of-processor-conditions)
   - [Error Handling](./processors.md#error-handling)
 - [Headers](./headers.md)
   - [Overview](./headers.md#overview)
   - [Existing **X-Include** Keys](./headers.md#existing-x-include-keys)
   - [Add new **X-Include** Key](./headers.md#add-new-x-include-key)
 - [Filters](./filters.md)
   - [Overview](./filters.md#overview)
   - [ComparisonFilter Filter](./filters.md#comparisonfilter-filter)
   - [Existing Filters](./filters.md#existing-filters)
   - [FilterInterface Interface](./filters.md#filterinterface-interface)
   - [CollectionAwareFilterInterface Interface](./filters.md#collectionawarefilterinterface-interface)
   - [MetadataAwareFilterInterface Interface](./filters.md#metadataawarefilterinterface-interface)
   - [RequestAwareFilterInterface Interface](./filters.md#requestawarefilterinterface-interface)
   - [SelfIdentifiableFilterInterface Interface](./filters.md#selfidentifiablefilterinterface-interface)
   - [NamedValueFilterInterface Interface](./filters.md#namedvaluefilterinterface-interface)
   - [StandaloneFilter Base Class](./filters.md#standalonefilter-base-class)
   - [StandaloneFilterWithDefaultValue Base Class](./filters.md#standalonefilterwithdefaultvalue-base-class)
   - [Criteria Class](./filters.md#criteria-class)
   - [CriteriaConnector Class](./filters.md#criteriaconnector-class)
   - [QueryExpressionVisitor Class](./filters.md#queryexpressionvisitor-class)
   - [Query Expressions](./filters.md#query-expressions)
   - [Creating New Filter](./filters.md#creating-new-filter)
   - [Other Classes](./filters.md#other-classes)
 - [How To](./how_to.md)
   - [Turn on API for an Entity](./how_to.md#turn-on-api-for-an-entity)
   - [Turn on API for an Entity Disabled in "Resources/config/oro/entity.yml"](./how_to.md#turn-on-api-for-an-entity-disabled-in-resourcesconfigoroentityyml)
   - [Enable Advanced Operators for String Filter](./how_to.md#enable-advanced-operators-for-string-filter)
   - [Enable Case-insensitive String Filter](./how_to.md#enable-case-insensitive-string-filter)
   - [Change an ACL Resource for an Action](./how_to.md#change-an-acl-resource-for-an-action)
   - [Disable Access Checks for an Action](./how_to.md#disable-access-checks-for-an-action)
   - [Disable an Entity Action](./how_to.md#disable-an-entity-action)
   - [Change the Delete Handler for an Entity](./how_to.md#change-the-delete-handler-for-an-entity)
   - [Change the Maximum Number of Entities that Can Be Deleted by One Request](./how_to.md#change-the-maximum-number-of-entities-that-can-be-deleted-by-one-request)
   - [Configure a Nested Object](./how_to.md#configure-a-nested-object)
   - [Configure a Nested Association](./how_to.md#configure-a-nested-association)
   - [Configure an Extended Many-To-One Association](./how_to.md#configure-an-extended-many-to-one-association)
   - [Configure an Extended Many-To-Many Association](./how_to.md#configure-an-extended-many-to-many-association)
   - [Configure an Extended Multiple Many-To-One Association](./how_to.md#configure-an-extended-multiple-many-to-one-association)
   - [Add a Custom Controller](./how_to.md#add-a-custom-controller)
   - [Add a Custom Route](./how_to.md#add-a-custom-route)
   - [Using a Non-primary Key to Identify an Entity](./how_to.md#using-a-non-primary-key-to-identify-an-entity)
   - [Enable API for an Entity Without Identifier](./how_to.md#enable-api-for-an-entity-without-identifier)
   - [Enable Custom API](./how_to.md#enable-custom-api)
   - [Add a Predefined Identifier for API Resource](./how_to.md#add-a-predefined-identifier-for-api-resource)
   - [Add a Computed Field](./how_to.md#add-a-computed-field)
   - [Disable HATEOAS](./how_to.md#disable-hateoas)
   - [Validate Virtual Fields](./how_to.md#validate-virtual-fields)
 - [CORS Configuration](./cors.md)
 - [Testing](./testing.md)
   - [Overview](./testing.md#overview)
   - [Load Fixtures](./testing.md#load-fixtures)
   - [Alice references](./testing.md#alice-references)
   - [Yaml templates](./testing.md#yaml-templates)
   - [Assert expectations](./testing.md#assert-expectations)
   - [Yaml templates for request body](./testing.md#yaml-templates-for-request-body)
   - [Process single reference](./testing.md#process-single-reference)
   - [Dump response into Yaml template](./testing.md#dump-response-into-yaml-template)
